@@ -16,12 +16,9 @@
 
 import numpy as np
 
-from nums.core.storage.storage import ArrayGrid
-from nums.core.systems.systems import System
 from nums.core.array import utils as array_utils
-
-
-block_id_counter = -1
+from nums.core.compute.compute_manager import ComputeManager
+from nums.core.grid.grid import ArrayGrid
 
 
 class Block(object):
@@ -29,9 +26,11 @@ class Block(object):
     # TODO(hme): Create a base class, and move this concrete class into blockarray.py.
     #  Do this when we implement a SparseBlock object.
 
-    def __init__(self, grid_entry, grid_shape, rect, shape, dtype, transposed, system: System,
+    block_id_counter = -1
+
+    def __init__(self, grid_entry, grid_shape, rect, shape, dtype, transposed, cm: ComputeManager,
                  id=None):
-        self._system = system
+        self._cm = cm
         self.grid_entry: tuple = grid_entry
         self.grid_shape: tuple = grid_shape
         self.rect: list = rect
@@ -42,12 +41,13 @@ class Block(object):
         self.transposed = transposed
         self.id = id
         if self.id is None:
-            global block_id_counter
-            block_id_counter += 1
-            self.id = block_id_counter
+            Block.block_id_counter += 1
+            self.id = Block.block_id_counter
+        # Set if a device id was used to compute this block.
+        self.device_id = None
 
     def __repr__(self):
-        return "Block("+str(self.oid)+")"
+        return "Block(" + str(self.oid) + ")"
 
     def size(self):
         return np.product(self.shape)
@@ -55,7 +55,7 @@ class Block(object):
     def copy(self, shallow=True):
         assert shallow, "Only shallow copies are currently supported."
         block = Block(self.grid_entry, self.grid_shape, self.rect, self.shape,
-                      self.dtype, self.transposed, self._system)
+                      self.dtype, self.transposed, self._cm)
         block.oid = self.oid
         return block
 
@@ -80,78 +80,56 @@ class Block(object):
                        shape=tuple(reversed(self.shape)),
                        dtype=self.dtype,
                        transposed=not self.transposed,
-                       system=self._system)
+                       cm=self._cm)
         blockT.oid = self.oid
         return blockT
 
-    def ufunc(self, op_name, options=None):
-        return self.uop_map(op_name, options=options)
+    def swapaxes(self, axis1, axis2):
+        block = self.copy()
+        grid_entry = list(block.grid_entry)
+        grid_shape = list(block.grid_shape)
+        shape = list(block.shape)
+        rect = block.rect
 
-    def uop_map(self, op_name, args=None, kwargs=None, options=None):
+        grid_entry[axis1], grid_entry[axis2] = grid_entry[axis2], grid_entry[axis1]
+        grid_shape[axis1], grid_shape[axis2] = grid_shape[axis2], grid_shape[axis1]
+        shape[axis1], shape[axis2] = shape[axis2], shape[axis1]
+        rect[axis1], rect[axis2] = rect[axis2], rect[axis1]
+
+        block.grid_entry = tuple(grid_entry)
+        block.grid_shape = tuple(grid_shape)
+        block.shape = tuple(shape)
+        block.rect = rect
+
+        block.oid = self._cm.swapaxes(block.oid, axis1, axis2,
+                                      syskwargs={
+                                          "grid_entry": block.grid_entry,
+                                          "grid_shape": block.grid_shape
+                                      })
+        return block
+
+    def ufunc(self, op_name, device_id=None):
+        return self.uop_map(op_name, device_id=device_id)
+
+    def uop_map(self, op_name, args=None, kwargs=None, device_id=None):
         # This retains transpose.
         block = self.copy()
         block.dtype = array_utils.get_uop_output_type(op_name, self.dtype)
         args = () if args is None else args
         kwargs = {} if kwargs is None else kwargs
-        if options is None:
-            block.oid = self._system.map_uop(op_name,
-                                             self.oid,
-                                             args,
-                                             kwargs,
-                                             syskwargs={
-                                                 "grid_entry": block.grid_entry,
-                                                 "grid_shape": block.grid_shape
-                                             })
+        if device_id is None:
+            syskwargs = {
+                "grid_entry": block.grid_entry,
+                "grid_shape": block.grid_shape
+            }
         else:
-            block.oid = self._system.call_with_options("map_uop",
-                                                       [op_name, self.oid,
-                                                        args,
-                                                        kwargs],
-                                                       {},
-                                                       options)
-        return block
-
-    def reduce_axis(self, op_name, axis, keepdims):
-        # TODO (hme): Add options version of this too, but need to fix block imps
-        #  so we're not branching between options / no options calls in every method.
-        # We lose an axis, so make sure to account for dropped axis in result.
-        result_grid_entry = []
-        result_grid_shape = []
-        result_rect = []
-        result_shape = []
-        for curr_axis in range(len(self.shape)):
-            if curr_axis == axis or axis is None:
-                if keepdims:
-                    result_grid_entry.append(0)
-                    result_grid_shape.append(1)
-                    result_rect.append((0, 1))
-                    result_shape.append(1)
-                continue
-            result_grid_entry.append(self.grid_entry[curr_axis])
-            result_grid_shape.append(self.grid_shape[curr_axis])
-            result_rect.append(self.rect[curr_axis])
-            result_shape.append(self.shape[curr_axis])
-
-        dtype = array_utils.get_reduce_output_type(op_name, self.dtype)
-        block = Block(grid_entry=result_grid_entry,
-                      grid_shape=result_grid_shape,
-                      rect=result_rect,
-                      shape=result_shape,
-                      dtype=dtype,
-                      # This is false because we invoke the transpose before
-                      # applying the reduction operation, so the resulting
-                      # remote object will be transposed.
-                      transposed=False,
-                      system=self._system)
-        block.oid = self._system.reduce_axis(op_name=op_name,
-                                             arr=self.oid,
-                                             axis=axis,
-                                             keepdims=keepdims,
-                                             transposed=self.transposed,
-                                             syskwargs={
-                                                 "grid_entry": block.grid_entry,
-                                                 "grid_shape": block.grid_shape
-                                             })
+            syskwargs = {"device_id": device_id}
+        block.device_id = device_id
+        block.oid = self._cm.map_uop(op_name,
+                                     self.oid,
+                                     args,
+                                     kwargs,
+                                     syskwargs=syskwargs)
         return block
 
     def _block_from_other(self, other):
@@ -160,11 +138,11 @@ class Block(object):
         # where a literal is used in the operation.
         assert isinstance(other, (int, float, np.int, np.float))
         block = Block(self.grid_entry, self.grid_shape, [(0, 1)], (1,),
-                      self.dtype, False, self._system)
-        block.oid = self._system.put(np.array(other, dtype=self.dtype))
+                      self.dtype, False, self._cm)
+        block.oid = self._cm.put(np.array(other, dtype=self.dtype))
         return block
 
-    def bop(self, op, other, args: dict, options=None):
+    def bop(self, op, other, args: dict, device_id=None):
         if not isinstance(other, Block):
             other = self._block_from_other(other)
         if op == "tensordot":
@@ -213,36 +191,23 @@ class Block(object):
                       shape=result_shape,
                       dtype=dtype,
                       transposed=False,
-                      system=self._system)
+                      cm=self._cm)
 
-        # TODO (hme): Get rid of requiring shape as param.
-        if options is None:
-            block.oid = self._system.bop(op,
-                                         self.oid,
-                                         other.oid,
-                                         self.shape,
-                                         other.shape,
-                                         self.transposed,
-                                         other.transposed,
-                                         axes=args.get("axes"),
-                                         syskwargs={
-                                             "grid_entry": block.grid_entry,
-                                             "grid_shape": block.grid_shape
-                                         })
+        if device_id is None:
+            syskwargs = {
+                "grid_entry": block.grid_entry,
+                "grid_shape": block.grid_shape
+            }
         else:
-            block.oid = self._system.call_with_options("bop",
-                                                       [
-                                                           op,
-                                                           self.oid,
-                                                           other.oid,
-                                                           self.shape,
-                                                           other.shape,
-                                                           self.transposed,
-                                                           other.transposed
-                                                       ], {
-                                                           "axes": args.get("axes")
-                                                       },
-                                                       options)
+            syskwargs = {"device_id": device_id}
+        block.device_id = device_id
+        block.oid = self._cm.bop(op,
+                                 self.oid,
+                                 other.oid,
+                                 self.transposed,
+                                 other.transposed,
+                                 axes=args.get("axes"),
+                                 syskwargs=syskwargs)
         return block
 
     def tensordot(self, other, axes):
@@ -294,12 +259,12 @@ class Block(object):
     def astype(self, dtype):
         block = self.copy()
         block.dtype = dtype
-        block.oid = self._system.astype(self.oid,
-                                        dtype.__name__,
-                                        syskwargs={
-                                            "grid_entry": block.grid_entry,
-                                            "grid_shape": block.grid_shape
-                                        })
+        block.oid = self._cm.astype(self.oid,
+                                    dtype.__name__,
+                                    syskwargs={
+                                        "grid_entry": block.grid_entry,
+                                        "grid_shape": block.grid_shape
+                                    })
         return block
 
     def conjugate(self):
@@ -308,25 +273,19 @@ class Block(object):
     def sqrt(self):
         return self.ufunc("sqrt")
 
-    def syskwargs(self):
-        # TODO (hme): This has a lot of potential scheduling bugs:
-        #  What if this block is transposed?
-        return {
-            "grid_entry": self.grid_entry,
-            "grid_shape": self.grid_shape
-        }
-
     def get(self):
-        return self._system.get(self.oid)
+        return self._cm.get(self.oid)
 
 
 class BlockArrayBase(object):
 
-    def __init__(self, grid: ArrayGrid, system: System, blocks: np.ndarray = None):
+    def __init__(self, grid: ArrayGrid, cm: ComputeManager, blocks: np.ndarray = None):
         self.grid = grid
-        self.system = system
+        self.cm = cm
         self.shape = self.grid.shape
         self.block_shape = self.grid.block_shape
+        self.size = np.product(self.shape)
+        self.ndim = len(self.shape)
         self.dtype = self.grid.dtype
         self.blocks = blocks
         if self.blocks is None:
@@ -340,23 +299,26 @@ class BlockArrayBase(object):
                                                 shape=self.grid.get_block_shape(grid_entry),
                                                 dtype=self.dtype,
                                                 transposed=False,
-                                                system=self.system)
+                                                cm=self.cm)
 
     def __repr__(self):
-        return "BlockArray("+str(self.blocks)+")"
+        return "BlockArray(" + str(self.blocks) + ")"
 
-    def get(self):
-        result = np.zeros(shape=self.grid.shape, dtype=self.grid.dtype)
-        block_shape = np.array(self.grid.block_shape, dtype=np.int)
-        blocks = self.system.get([self.blocks[grid_entry].oid
-                                  for grid_entry in self.grid.get_entry_iterator()])
+    def get(self) -> np.ndarray:
+        result: np.ndarray = np.zeros(shape=self.grid.shape, dtype=self.grid.dtype)
+        block_shape: np.ndarray = np.array(self.grid.block_shape, dtype=np.int)
+        arrays: list = self.cm.get([self.blocks[grid_entry].oid
+                                    for grid_entry in self.grid.get_entry_iterator()])
         for block_index, grid_entry in enumerate(self.grid.get_entry_iterator()):
             start = block_shape * grid_entry
             entry_shape = np.array(self.grid.get_block_shape(grid_entry), dtype=np.int)
             end = start + entry_shape
             slices = tuple(map(lambda item: slice(*item), zip(*(start, end))))
-            block = self.blocks[grid_entry]
-            result[slices] = blocks[block_index].reshape(block.shape)
+            block: Block = self.blocks[grid_entry]
+            arr: np.ndarray = arrays[block_index]
+            if block.transposed:
+                arr = arr.T
+            result[slices] = arr.reshape(block.shape)
         return result
 
     def broadcast_to(self, shape):
@@ -364,7 +326,7 @@ class BlockArrayBase(object):
         result_block_shape = array_utils.broadcast_block_shape(self.shape, shape, self.block_shape)
         result: BlockArrayBase = BlockArrayBase(ArrayGrid(b.shape,
                                                           result_block_shape,
-                                                          self.grid.dtype.__name__), self.system)
+                                                          self.grid.dtype.__name__), self.cm)
         extras = []
         # Below taken directly from _broadcast_to in numpy's stride_tricks.py.
         it = np.nditer(
